function vhb_mcmc(params, model, options, Rdata, ds, type)
% Written by Paul Miles            November 15, 2013
% Modefied by PM                   February 15, 2016
fprintf('**********************************************\n');
fprintf('Program Begin - %s\n\n',datestr(now));

% Define OS Delimiter
[delim] = OS_delimiter;

% -------------------------------------------------------------------------
% MTS Settings
Lambda_vec = [0.0472 0.10 0.335 0.50 0.67 6.7e-5]; % Hz - Lambda rate
Lambda = Lambda_vec(ds);
SR_vec = {'04' '10' '33' '50' '67' '00'};
SR = SR_vec{ds};

% -------------------------------------------------------------------------
% Define data structure
data.ydata = Rdata.ydata{ds}(:); % kPa
data.xdata = Rdata.xdata{ds}(:);
data.tinc = Rdata.tinc{ds};
data.ind = Rdata.Ind(ds,:);
data.type = type; % Sample all parameters - 1; Sample viscoelastic parameters - 2;

% -------------------------------------------------------------------------
% BAYESIAN ANALYSIS
[results, chain, s2chain, sschain] = mcmcrun(model,data,params,options);

chainstats(chain,results)

modelfun = @(d,th) vhb_model(th,d); % NOTE: the order in which d and th appear is important

nsample = 500;
out = mcmcpred(results,chain,s2chain,data,modelfun,nsample);

% -------------------------------------------------------------------------
% Calculate Error
[ss] = ss_func(results.mean,data);
Total_Error = ss/length(data.ydata);

fprintf('dL/dt = %8.4e\nError = %8.4f\n', Lambda, Total_Error);

% -------------------------------------------------------------------------
% Define output directory
OD.main = 'Results';
make_directory(OD.main);
OD.sub{1} = datestr(now,'YYYYmmdd_HHMMSS');
OD.sub{2} = strcat('SR_', SR, '_nsimu_', num2str(results.nsimu));
OD.path = strcat(OD.main, delim, OD.sub{1}, '_', OD.sub{2});
make_directory(OD.path);

% -------------------------------------------------------------------------
% Save results
filename = strcat(OD.sub{1}, '_', OD.sub{2});
save(filename)
move_files(filename, OD.path);

fprintf('Saving file: %s.mat\n', filename);
fprintf('Files saved to: %s\n', OD.path);

return
