function [sigma, sigma_inf, q] = vhb_model(theta,data)

% Define input arguments
stretch = data.xdata(:);
ind = data.ind(:);
dta = data.dta;

%% Hyperelastic Stress
% Parameters
Gc = theta(1);          % [kPa]
lam_max = theta(2);     % [-]
Ge = theta(3);          % [kPa]

% Stretch Invariant
I1 = stretch.^2 + 2./stretch;

%% Hyperelastic Stress
p = (Gc/3./stretch.*((9*lam_max^2 - I1)./(3*lam_max^2 - I1))) +...
    Ge./stretch.^0.5.*(1 - stretch);

% 1st P-K stress in Pa
Hc = 1./3.*Gc.*stretch.*((9*lam_max^2 - I1)./(3*lam_max^2 - I1));
He = Ge.*(1-1./stretch.^2);
sigma_inf = Hc + He - p./stretch; 

%% Viscoelastic Stress
% Parameters
gamma = theta(4);   % [kPa]
eta = theta(5);     % [kPa*s];
beta = theta(6);    % [-];

tau = eta/gamma; % viscoelastic time constant - might be off by factor of 2

n=length(stretch);
Q = zeros(n,1); % Initial viscoelastic value
dS_dlam = zeros(n,1); % Initial viscoelastic value
dS_dlam(1) = (sigma_inf(2) - sigma_inf(1))/(stretch(2) - stretch(1));

p = 1:1:n;
ds = setdiff(p,ind+1); % removes starting indices
for k = 1:length(ds);
    i = ds(k);
    Tnc = 1 - dta(i)/(2*tau);
    Tpc = 1 + dta(i)/(2*tau);
    Tpcinv = Tpc.^(-1);
    
    Q(i) = Tpcinv.*(Tnc.*Q(i-1) + beta*(sigma_inf(i) - sigma_inf(i-1)));
    
    % Numerically calculate the hyperelastic stress slope
    dS_dlam(i) = (sigma_inf(i) - sigma_inf(i-1))/(stretch(i) - stretch(i-1));
end
% Check for singularities
sing_ind = find(isnan(dS_dlam));
dS_dlam(sing_ind) = dS_dlam(sing_ind-1);

% Internal state variable
Gamma = 1/gamma*(beta*sigma_inf-Q);

% Viscoelastic stress
q = beta*(sigma_inf - (dS_dlam(:)).*Gamma(:)); 

%% Nominal stress
sigma_inf = sigma_inf(:); % [kPa]
q = q(:); % [kPa]
sigma = (sigma_inf + q); % [kPa]